import { ChangeEvent, useEffect, useRef, useState } from "react";
import { apiAuth } from "./api";
import { GraphCanvas, GraphCanvasRef, GraphEdge, GraphNode, recommendLayout, useSelection } from "reagraph";
import { customTheme } from "./NetworkTheme";

interface NetworkData {
  nodes: GraphNode[],
  edges: GraphEdge[],
}

export const GraphComponent = () => {
  const [data, setData] = useState({ nodes: [], edges: [] } as NetworkData);
  const [loading, setLoading] = useState(true);
  const [threed, setThreed] = useState(false);

  async function loadData() {
    setLoading(true);
    await apiAuth("analysis/graph_json", null)
      .then(json => json as Promise<NetworkData>).then(json => { setData(json) })
    setLoading(false);
  }

  useEffect(() => { loadData() }, [])

  const graphRef = useRef<GraphCanvasRef | null>(null);

  const { selections, actives, onNodeClick, onCanvasClick } = useSelection({
    ref: graphRef,
    nodes: data.nodes,
    edges: data.edges,
    pathSelectionType: 'out',
  });

  function handleThreed() {
    setThreed(!threed)
    graphRef.current?.fitNodesInView();
    graphRef.current?.centerGraph();
    graphRef.current?.resetControls();
  }

  return (
    <div style={{ position: 'absolute', top: 0, bottom: 0, left: 0, right: 0 }}>
      <div className="controls" >
        <div className="control" onClick={handleThreed}>
          <span>2D</span>
          <div className="switch">
            <input type="checkbox" checked={threed} />
            <span className="slider round"></span>
          </div>
          <span>3D</span>
        </div>
      </div>
      <GraphCanvas
        draggable
        cameraMode={threed ? "rotate" : "pan"}
        layoutType={threed ? "forceDirected3d" : "forceDirected2d"}
        layoutOverrides={{
          linkDistance: 200
        }}
        labelType="auto"
        ref={graphRef}
        theme={customTheme}
        nodes={data.nodes}
        edges={data.edges}
        selections={selections}
        actives={actives}
        onCanvasClick={onCanvasClick}
        onNodeClick={onNodeClick}
      />
    </div>
  );

}

